@@ -1,6 +1,6 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
-* Copyright 2013 - 2021, nymea GmbH
+* Copyright 2013 - 2025, nymea GmbH
 * Contact: contact@nymea.io
 *
 * This fileDescriptor is part of nymea.
@@ -34,6 +34,7 @@
 SunSpecLfrtcModelRepeatingBlock::SunSpecLfrtcModelRepeatingBlock(quint16 blockIndex, quint16 blockSize, quint16 modbusStartRegister, SunSpecLfrtcModel *parent) :
     SunSpecModelRepeatingBlock(blockIndex, blockSize, modbusStartRegister, parent)
 {
+    m_parentModel = parent;
     m_byteOrder = parent->byteOrder();
     initDataPoints();
 }
@@ -1326,10 +1327,8 @@ void SunSpecLfrtcModelRepeatingBlock::initDataPoints()
 
 }
 
-void SunSpecLfrtcModelRepeatingBlock::processBlockData(const QVector<quint16> blockData)
+void SunSpecLfrtcModelRepeatingBlock::processBlockData()
 {
-    m_blockData = blockData;
-
     // Update properties according to the data point type
     if (m_dataPoints.value("ActPt").isValid())
         m_actPt = m_dataPoints.value("ActPt").toUInt16();
@@ -1471,6 +1470,8 @@ SunSpecLfrtcModel::SunSpecLfrtcModel(SunSpecConnection *connection, quint16 modb
     m_modelBlockType = SunSpecModel::ModelBlockTypeFixedAndRepeating;
 
     initDataPoints();
+
+    connect(this, &SunSpecModel::initFinished, this, &SunSpecLfrtcModel::setupRepeatingBlocks);
 }
 
 SunSpecLfrtcModel::~SunSpecLfrtcModel()
@@ -1796,6 +1797,36 @@ void SunSpecLfrtcModel::processBlockData()
     qCDebug(dcSunSpecModelData()) << this;
 }
 
+void SunSpecLfrtcModel::setupRepeatingBlocks()
+{
+    if (!m_repeatingBlocks.isEmpty()) {
+        foreach (SunSpecModelRepeatingBlock *block, m_repeatingBlocks) {
+            block->deleteLater();
+        }
+        m_repeatingBlocks.clear();
+    }
+
+    const auto headerLength = 2;
+    const auto repeatingBlocksDataSize = m_blockData.size() - headerLength - m_fixedBlockLength;
+    if (repeatingBlocksDataSize % m_repeatingBlockLength != 0) {
+        qCWarning(dcSunSpecModelData()) << "Unexpected repeating block data size:"
+                                        << repeatingBlocksDataSize
+                                        << "(repeating block size:"
+                                        << m_repeatingBlockLength
+                                        << ", extra bytes:"
+                                        << repeatingBlocksDataSize % m_repeatingBlockLength
+                                        << "). Repeating blocks will not be handled!";
+        return;
+    }
+    const auto numberOfBlocks = repeatingBlocksDataSize / m_repeatingBlockLength;
+    const auto repeatingBlocksOffset = m_fixedBlockLength + headerLength;
+    for (int i = 0; i < numberOfBlocks; ++i) {
+        const auto blockStartRegister = static_cast<quint16>(modbusStartRegister() + repeatingBlocksOffset + m_repeatingBlockLength * i);
+        const auto block = new SunSpecLfrtcModelRepeatingBlock(i, m_repeatingBlockLength, blockStartRegister, this);
+        m_repeatingBlocks.append(block);
+    }
+}
+
 QDebug operator<<(QDebug debug, SunSpecLfrtcModel *model)
 {
     debug.nospace().noquote() << "SunSpecLfrtcModel(Model: " << model->modelId() << ", Register: " << model->modbusStartRegister() << ", Length: " << model->modelLength() << ")\n";
